/************************************************************************

ConfigCallbacks.cpp

Copyright (C) 2007 Virus
Copyright (C) 2002 mooby

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

************************************************************************/

#pragma warning(disable:4786)

#include "Utils.hpp"
#include "FileInterface.hpp"
#include "Preferences.hpp"
#include "ConfigCallbacks.hpp"
#include "RipCDDAData.hpp"

#include <fstream>
#include <string>
#include <vector>
#include <sstream>

using namespace std;

extern Preferences prefs;

// compresses cd using the compression methods in zipCD
void compressIt(FileInterface* cd, 
        CompressedFileInterface* zipCD, 
        const string& outputFileName,
        const string& indexFileName,
        GtkProgressBar* prog)
{
    cd->setCacheMode(FileInterface::oldMode);
    CDTime start(0,2,0);
    // these hold the info needed for generating the .index or .table file
    vector<unsigned long> indexTable;
    vector<unsigned long> sizeTable;

    // open the output file
    std::ofstream outfile(outputFileName.c_str(), ios::binary);

    // the buffers for compression
    unsigned char* inputBuffer = 
        new unsigned char[zipCD->getCompressedFrames() * bytesPerFrame];
    unsigned char* outputBuffer = 
        new unsigned char[(zipCD->getCompressedFrames() + 1) * bytesPerFrame];

    gtk_progress_bar_set_text(prog, "Compressing...");

    while (start < cd->getCDLength())
    {
        unsigned int i = 0;
        // read up to compressedFrames or until the end of the CD, whichever is smaller
        while ( (start < cd->getCDLength()) && (i < zipCD->getCompressedFrames()) )
        {
            gtk_progress_bar_set_fraction(prog,(float)start.getAbsoluteFrame() / 
                    (float)cd->getCDLength().getAbsoluteFrame());
            while (gtk_events_pending()) gtk_main_iteration();
            cd->seek(start);
            memcpy(inputBuffer + bytesPerFrame * i, cd->getBuffer(), bytesPerFrame);
            i++;
            start += CDTime(0,0,1);
        }
        // add the index information for the file location
        indexTable.push_back(outfile.tellp());

        // compress the data
        unsigned int outputBytes = (zipCD->getCompressedFrames() + 1) * bytesPerFrame;
        unsigned int inputBytes = i * bytesPerFrame;
        zipCD->compressData((char*)inputBuffer, (char*)outputBuffer, 
                inputBytes, outputBytes);

        // write the compressed data and add the compressed size to the sizeTable
        outfile.write((char*)outputBuffer, outputBytes);
        sizeTable.push_back(outputBytes);
    }

    // write the index file
    string tableToWrite = zipCD->toTable(indexTable, sizeTable);

    ofstream indexFile(indexFileName.c_str(), ios::binary);
    indexFile.write(tableToWrite.c_str(), tableToWrite.size());

    // clean up
    indexFile.close();
    outfile.close();
    delete [] inputBuffer;
    delete [] outputBuffer;
    delete cd;
    delete zipCD;

    gtk_progress_bar_set_text(prog, "Progress");
    moobyMessage("Done");
}

// decompressing is much easier
void decompressIt(FileInterface* cd, 
        const string& outputFileName,
        GtkProgressBar* prog)
{
    cd->setCacheMode(FileInterface::oldMode);
    CDTime start(0,2,0);
    std::ofstream outfile(outputFileName.c_str(), ios::binary);

    gtk_progress_bar_set_text(prog, "Decompressing...");

    // write out every frame in the file
    while (start < cd->getCDLength())
    {
        gtk_progress_bar_set_fraction(prog, (float)start.getAbsoluteFrame() / 
            (float)cd->getCDLength().getAbsoluteFrame());
        while(gtk_events_pending()) gtk_main_iteration();
        cd->seek(start);
        outfile.write((char*)cd->getBuffer(), bytesPerFrame);
        start += CDTime(0,0,1);
    }

    outfile.close();
    delete cd;

    gtk_progress_bar_set_text(prog, "Progress");
    moobyMessage("Done");
}

void compress_Z_cb(GtkWidget* widget, gpointer user_data)
{
    char * returned;
    if ( (returned = moobyFileChooser("Choose a file to compress in "
                    "Z.table format", NULL)) == NULL)
        return;
    string outputFileName(returned);
    outputFileName += ".Z";
    string indexFileName(outputFileName + string(".table"));
    FileInterface* cd = new UncompressedFileInterface(1);
    CompressedFileInterface* zipCD = new ZTableFileInterface(1);
    cd->openFile(returned);
    compressIt(cd, zipCD, outputFileName, indexFileName, GTK_PROGRESS_BAR(user_data));
}

void decompress_Z_cb(GtkWidget* widget, gpointer user_data)
{
    char * returned;
    if ( (returned = moobyFileChooser("Choose a .Z file to decompress", "*.Z")) == NULL)
        return;
    FileInterface* cd = new ZTableFileInterface(1);
    string theFile(returned);
    cd->openFile(theFile);
    string outputFileName(theFile);
    outputFileName.erase(theFile.rfind(".Z"));
    decompressIt(cd, outputFileName, GTK_PROGRESS_BAR(user_data));
}

void compress_bz_cb(GtkWidget* widget, gpointer user_data)
{
    char * returned;
    if ( (returned = moobyFileChooser("Choose a file to compress in "
                    "bz.index format", NULL)) == NULL)
        return;
    string outputFileName(returned);
    outputFileName += ".bz";
    string indexFileName(outputFileName + string(".index"));
    FileInterface* cd = new UncompressedFileInterface(1);
    CompressedFileInterface* zipCD = new BZIndexFileInterface(1);
    cd->openFile(returned);
    compressIt(cd, zipCD, outputFileName, indexFileName, GTK_PROGRESS_BAR(user_data));
}

void decompress_bz_cb(GtkWidget* widget, gpointer user_data)
{
    char * returned;
    if ( (returned = moobyFileChooser("Choose a .bz file to decompress", "*.bz")) == NULL)
        return;
    FileInterface* cd = new BZIndexFileInterface(1);
    string theFile(returned);
    cd->openFile(theFile);
    string outputFileName(theFile);
    outputFileName.erase(theFile.rfind(".bz"));
    decompressIt(cd, outputFileName, GTK_PROGRESS_BAR(user_data));
}

void close_cb(GtkWidget *widget, gpointer user_data) {
	gtk_widget_destroy(GTK_WIDGET(user_data));
    gtk_main_quit();
}

void volume_cb(GtkWidget *widget, gpointer)
{
    gchar* tmp = g_strdup_printf("%f",gtk_range_get_value(GTK_RANGE(widget)));
    prefs.prefsMap[volumeString] = std::string(tmp);
    g_free(tmp);
}

void cache_cb(GtkWidget* widget, gpointer)
{
    gchar* tmp = g_strdup_printf("%d",(int)gtk_range_get_value(GTK_RANGE(widget)));
    prefs.prefsMap[cacheSizeString] = std::string(tmp);
    g_free(tmp);
}

void read_subchannel_cb(GtkWidget* widget, gpointer)
{
    prefs.prefsMap[subEnableString] =  
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) 
        ? std::string("1") : std::string();
}

void alternative_caching_cb(GtkWidget* widget, gpointer)
{
    prefs.prefsMap[cachingModeString] =  
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) 
        ? std::string(newCachingString) : std::string(oldCachingString);

}

// Radio buttons for repetition configuration
void repeat_all_cb(GtkWidget* widget, gpointer)
{ prefs.prefsMap[repeatString] = std::string(repeatAllString); }
void repeat_one_cb(GtkWidget* widget, gpointer)
{ prefs.prefsMap[repeatString] = std::string(repeatOneString); }
void play_one_cb(GtkWidget* widget, gpointer)
{ prefs.prefsMap[repeatString] = std::string(playOneString); }

void auto_load_cb(GtkWidget* widget, gpointer user_data)
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
        gchar* file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(user_data));
        if (file)
            prefs.prefsMap[autorunString] = std::string(file);
        else
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),false);
    }
    else
    {
        prefs.prefsMap[autorunString] = std::string();
        gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(user_data));
    }
}

void auto_load_file_cb(GtkWidget* widget, gpointer user_data)
{
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data),true); 
    auto_load_cb(GTK_WIDGET(user_data),widget);
}

void rip_wav_cb(GtkWidget* widget, gpointer user_data)
{
    gpointer* two = (gpointer*)user_data;
    GtkProgressBar* prog = GTK_PROGRESS_BAR(two[1]);
    gchar* file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(two[0]));
    if (file)
    {
        RipCDDAData* ripper = new RipCDDAData(file,prog);
        if (ripper->hasCDDAData())
        {
            ripper->doRip();
            delete ripper;
        }
        else
            moobyMessage("No CDDA data in this image!\n");
    }
    else
        moobyMessage("No image selected!\n");
}

void clear_cdda_cb(GtkWidget* widget, gpointer user_data)
{
    gpointer* two = (gpointer*)user_data;
    GtkProgressBar* prog = GTK_PROGRESS_BAR(two[1]);
    gchar* file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(two[0]));
    if (file)
    {
        RipCDDAData* ripper = new RipCDDAData(file,prog);
        if (ripper->hasCDDAData())
        {
            ripper->clearCDDA();
            delete ripper;
        }
        else
            moobyMessage("No CDDA data in this image!\n");
    }
    else
        moobyMessage("No image selected!\n");
}

