/************************************************************************

ConfigFunctions.cpp

Copyright (C) 2007 Virus
Copyright (C) 2002 mooby

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

************************************************************************/

#include "defines.h"
#include "externs.h"
#include "Utils.hpp"
#include "Preferences.hpp"
#include "ConfigCallbacks.hpp"
#include <glade/glade.h>

extern Preferences prefs;

#define GTKCONNECT(name,event,ud) \
	GtkWidget* name = glade_xml_get_widget(xml, #name); \
	g_signal_connect_data(GTK_OBJECT(name), event, \
		GTK_SIGNAL_FUNC(name ## _cb), ud, NULL, G_CONNECT_AFTER);

long CALLBACK CDRconfigure(void)
{
	GladeXML *xml = glade_xml_new(DATADIR "dfbinimage.glade2", "config", NULL);
    if (!xml) {
		g_warning("We could not load the interface! (dfbinimage.glade2)");
		return -1; // Don't know what error code i should use
    }
	GtkWidget* config = glade_xml_get_widget(xml, "config");
	glade_xml_signal_autoconnect (xml);

	GtkWidget* auto_load_file  = glade_xml_get_widget(xml,"auto_load_file");
	GtkWidget* filechooser_wav = glade_xml_get_widget(xml,"filechooser_wav");
	GtkWidget* progress_Z      = glade_xml_get_widget(xml,"progress_Z");
	GtkWidget* progress_bz     = glade_xml_get_widget(xml,"progress_bz");
	GtkWidget* progress_wav    = glade_xml_get_widget(xml,"progress_wav");
    // Connect all the callback functions
    GTKCONNECT(close,              "clicked",(gpointer)config);
    GTKCONNECT(compress_Z,         "clicked",progress_Z);
    GTKCONNECT(decompress_Z,       "clicked",progress_Z);
    GTKCONNECT(compress_bz,        "clicked",progress_bz);
    GTKCONNECT(decompress_bz,      "clicked",progress_bz);
    GTKCONNECT(read_subchannel,    "toggled",NULL);
    GTKCONNECT(alternative_caching,"toggled",NULL);
    GTKCONNECT(repeat_all,         "clicked",NULL);
    GTKCONNECT(repeat_one,         "clicked",NULL);
    GTKCONNECT(play_one,           "clicked",NULL);
    GTKCONNECT(cache,              "change-value",NULL);
    GTKCONNECT(volume,             "change-value",NULL);

    gpointer two[2] = {filechooser_wav,progress_wav};
    GTKCONNECT(rip_wav,            "clicked",(gpointer)two);
    GTKCONNECT(clear_cdda,         "clicked",(gpointer)two);

    GTKCONNECT(auto_load,"toggled",(gpointer)auto_load_file);
    g_signal_connect_data(GTK_OBJECT(auto_load_file), "file-set",
		GTK_SIGNAL_FUNC(auto_load_file_cb), auto_load, NULL, G_CONNECT_AFTER);

    // Add filter to the file browser
    GtkFileFilter *filter = gtk_file_filter_new ();
    GtkFileFilter *filterAll = gtk_file_filter_new ();
    gtk_file_filter_add_pattern (filterAll,"*");
    gtk_file_filter_set_name (filterAll,"All Files");
    gtk_file_filter_set_name (filter,"Supported files");
    gtk_file_filter_add_pattern (filter,"*.iso");
    gtk_file_filter_add_pattern (filter,"*.bin");
    gtk_file_filter_add_pattern (filter,"*.bwi");
    gtk_file_filter_add_pattern (filter,"*.img");
    gtk_file_filter_add_pattern (filter,"*.Z");
    gtk_file_filter_add_pattern (filter,"*.Z.table");
    gtk_file_filter_add_pattern (filter,"*.bz");
    gtk_file_filter_add_pattern (filter,"*.bz.index");
    gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(auto_load_file), filterAll);
    gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(auto_load_file), filter);
    gtk_file_chooser_set_filter( GTK_FILE_CHOOSER(auto_load_file), filter);

    gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(filechooser_wav), filterAll);
    gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(filechooser_wav), filter);
    gtk_file_chooser_set_filter( GTK_FILE_CHOOSER(filechooser_wav), filter);

    // Apply current settings
    // subchannel usage
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(read_subchannel), 
        prefs.prefsMap[subEnableString].compare("1")==0);
    // alternative caching
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alternative_caching), 
        prefs.prefsMap[cachingModeString].compare(newCachingString)==0);
    // repeat
    if (prefs.prefsMap[repeatString].compare(repeatAllString)==0)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(repeat_all),true);
    else if (prefs.prefsMap[repeatString].compare(repeatOneString)==0)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(repeat_one),true);
    else //if (prefs.prefsMap[repeatString].compare(playOneString)==0)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(play_one),true);
    // volume
    double vol = 1;
    { std::istringstream tmp(prefs.prefsMap[volumeString]); tmp >> vol; }
    gtk_range_set_value(GTK_RANGE(volume),vol);
    // cache
    int size = 1;
    { std::istringstream tmp(prefs.prefsMap[cacheSizeString]); tmp >> size; }
    gtk_range_set_value(GTK_RANGE(cache),size);

    if (prefs.prefsMap[autorunString].length() > 0)
    {
        gboolean exists = gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(auto_load_file),
            prefs.prefsMap[autorunString].c_str());
        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(auto_load_file),
            prefs.prefsMap[autorunString].c_str());
        while(gtk_events_pending()) gtk_main_iteration();
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(auto_load),true);
    }
    
	gtk_widget_show (config);
    gtk_main ();
	prefs.write();
	return 0;
}

int	CD_Configure(UINT32 *par)
{
    return CDRconfigure();
}

void CALLBACK CDVDconfigure()
{
   CDRconfigure();
}
