/************************************************************************

Preferences.cpp

Copyright (C) 2007 Virus
Copyright (C) 2002 mooby

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 ************************************************************************/

#include "Preferences.hpp"

Preferences::Preferences()
{
    allPrefs.push_back(volumeString);
    allPrefs.push_back(repeatString);
    allPrefs.push_back(autorunString);
    allPrefs.push_back(lastrunString);
    allPrefs.push_back(cacheSizeString);
    allPrefs.push_back(cachingModeString);
    allPrefs.push_back(subEnableString);

    // win32 code kindly pilfered from PEOPS =)
#ifdef WIN32
    const size_t bufSize = 1024;
    char* buffer = new char[bufSize];
    HKEY myKey;
    DWORD type;
    DWORD size;

    if (RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Vision Thing\\PSEmu Pro\\CDR\\MoobyCDR",
                0,KEY_ALL_ACCESS,&myKey)==ERROR_SUCCESS)
    {
        std::list<std::string>::iterator itr = allPrefs.begin();
        while (itr != allPrefs.end())
        {      
            size = bufSize;
            type = REG_SZ;
            if(RegQueryValueEx(myKey,(*itr).c_str(),0,&type,(LPBYTE)buffer,&size)==ERROR_SUCCESS)
                prefsMap[*itr] = std::string(buffer);
            else
                prefsMap[*itr] = std::string("");
            itr++;
        }
        RegCloseKey(myKey);
    }
    delete [] buffer;

#else
    GKeyFile *key_file = g_key_file_new();
    gboolean gkeyok = g_key_file_load_from_file(key_file, ("dfbinimage.cfg"),
        G_KEY_FILE_NONE, NULL);
    if (gkeyok)
    {
        std::list<std::string>::iterator itr = allPrefs.begin();
        while (itr != allPrefs.end())
        {
            prefsMap[*itr] = std::string("");
            if (gkeyok)
            {
                gchar *value = g_key_file_get_value(key_file,"DFBinImage",itr->c_str(),NULL);
                if (value)
                {
                    prefsMap[*itr] = std::string(value);
                    g_free(value);
                }
            }
            itr++;
        }
        g_key_file_free(key_file);
    }
#endif
}

void Preferences::write()
{
#ifdef WIN32
    HKEY myKey;
    DWORD myDisp;

    RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\Vision Thing\\PSEmu Pro\\CDR\\MoobyCDR",
            0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&myKey,&myDisp);

    std::map<std::string, std::string>::iterator itr = prefsMap.begin();
    while (itr != prefsMap.end())
    {
        RegSetValueEx(myKey,(itr->first).c_str(),0,REG_SZ,
                (LPBYTE) (itr->second).c_str(),itr->second.size() + 1);
        itr++;
    }
    RegCloseKey(myKey);

#else
    GKeyFile *key_file = g_key_file_new();
    gboolean gkeyok = g_key_file_load_from_file(key_file, ("dfbinimage.cfg"),
        G_KEY_FILE_NONE, NULL);
    std::map<std::string, std::string>::iterator itr = prefsMap.begin();
    while (itr != prefsMap.end())
    {
        g_key_file_set_value(key_file,"DFBinImage",(itr->first).c_str(),(itr->second).c_str());
        itr++;
    }
    gsize size;
    gchar* data = g_key_file_to_data(key_file,&size,NULL);
    g_file_set_contents(("dfbinimage.cfg"), data, size, NULL);
    g_free(data);
#endif
}

