/************************************************************************

RipCDDAData.cpp

Copyright (C) 2008 Micket

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

************************************************************************/
#include "RipCDDAData.hpp"

using namespace std;

extern Preferences prefs;

RipCDDAData::RipCDDAData(string file, GtkProgressBar* prog)
    : filename(file), prog(prog)
{
    printf("RipCDDAData::RipCDDAData\n");
    std::string extension;
    theCD = FileInterfaceFactory(file, extension);
    //theCD->setPregap(pregapLength, trackList[2].trackStart);
    string fileroot = filename;
    fileroot.erase(fileroot.rfind(extension));
    TrackParser* tp = TrackParserFactory(fileroot, theCD);
    tp->parse();
    tp->postProcess(theCD->getCDLength());
    trackList = tp->getTrackList();
    delete tp;
}

bool RipCDDAData::doRip()
{
    if (!hasCDDAData())
    {
        moobyMessage("There is no CDDA data in this image!\n");
        return false;
    }
    bool ok = true;
    for (int i = 2; i < trackList.size(); i++)
        ok = ok && saveSong(i);
}

bool RipCDDAData::saveSong(int track)
{
    char WAVHeader[] = {
        // Header chunk (12 bytes)
        'R','I','F','F',
        0,0,0,0,            // Data size, Modified later
        'W','A','V','E',
        // Format chunk (24 bytes)
        'f','m','t',' ',
        0x10,0,0,0,
        0x01,0,
        0x02,0,             // Stereo
        0x44,0xAC,0,0,      // 44.1 kHz
        0x10,0xB1,0x02,0,   // Bytes per second
        0x04,0,             // 16 bit stereo
        0x10,0,             // 16 Bits per sample
        // Data chunk
        'd','a','t','a',
        0,0,0,0,            // Samples, modified later
        };

    gchar* tmp = g_strdup_printf("%s.%02d.wav",filename.c_str(),track-1);
    string songname = tmp;
    gchar* tmp2 = g_path_get_basename(tmp);
    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prog),tmp2);
    g_free(tmp2);
    g_free(tmp);
    printf("RipCDDAData::saveSong - ripping %s\n",songname.c_str());
    CDTime CDDAStart = trackList[track].trackStart;
    cout << "START: " << CDDAStart;
    CDTime CDDAEnd   = trackList[track].trackEnd;
    CDTime CDDALength = trackList[track].trackLength;
    CDTime CDDAPos = CDDAStart;
    int samples  = CDDALength.getAbsoluteByte()/sizeof(short);
    int dataSize = CDDALength.getAbsoluteByte() + sizeof(WAVHeader) - 8;
    *(int*)(&WAVHeader[WAV_DATASIZE]) = GINT32_TO_LE(dataSize);
    *(int*)(&WAVHeader[WAV_SAMPLES])  = GINT32_TO_LE(samples*sizeof(short));
    FILE* fp = fopen(songname.c_str(),"wb");
    fwrite(WAVHeader,1,sizeof(WAVHeader),fp);
    int frameOffset = 0;
    int bufferSize = samplesPerFrame*framesPerSecond*5; // 5 seconds at the time
    short WAVData[bufferSize];
    bool stop = false;
    theCD->seek(CDDAPos);
    short* buffer = ((short*)theCD->getBuffer());
    while (!stop)
    {
        int i;
        for (i = 0; i < bufferSize && !stop; )
        {
            // It is safe to take 2 samples at the time, (Stereo)
            // This isn't super efficient, but it doesn't need to be either.
            WAVData[i++] = GINT16_TO_LE(buffer[frameOffset++]);
            WAVData[i++] = GINT16_TO_LE(buffer[frameOffset++]);
            if (frameOffset == samplesPerFrame)
            {
                CDDAPos += CDTime(0,0,1);
                if (CDDAPos <= CDDAEnd)
                {
                    theCD->seek(CDDAPos);
                    frameOffset = 0;
                    buffer = (short*)theCD->getBuffer();
                }
                else
                    stop = true;
            }
        }
        float frac = ((float)((CDDAPos-CDDAStart).getAbsoluteByte()))
              /CDDALength.getAbsoluteByte();
        if (frac > 1) frac = 1;
        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(prog),frac);
        while (gtk_events_pending()) gtk_main_iteration();
        //printf("Writing %d samples (max = %d), stop = %d, %f\n",i,bufferSize,stop,frac);
        fwrite(WAVData,sizeof(short),i,fp);
    }
    printf("Done writing samples, closing file\n");
    fclose(fp);
    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prog),"Done");
    return true;
}

bool RipCDDAData::clearCDDA()
{
    if (!hasCDDAData())
        return false;
    string oldFilename = filename+".backup";
    string newFilename = filename+".new";
    CDTime CDDALength = trackList[1].trackLength;
    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prog),"Copying game data");
    while (gtk_events_pending()) gtk_main_iteration();
    // Computers nowdays has enough room for this in one step.
    void* fileBuffer = malloc(CDDALength.getAbsoluteByte());
    FILE* fp = fopen(filename.c_str(),"rb");
    fread(fileBuffer,1,CDDALength.getAbsoluteByte(),fp);
    fclose(fp);
    fp = fopen(newFilename.c_str(),"wb");
    fwrite(fileBuffer,1,CDDALength.getAbsoluteByte(),fp);
    fclose(fp);
    // Now move the files
    g_rename(filename.c_str(),oldFilename.c_str());
    g_rename(newFilename.c_str(),filename.c_str());
    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prog),"Done");
    return true;
}
