#include "Utils.hpp"
#include "stdio.h"

void moobyMessage(const std::string& message)
{
    GtkWidget *dialog = gtk_message_dialog_new (NULL,
        GTK_DIALOG_MODAL,
        GTK_MESSAGE_INFO,
        GTK_BUTTONS_CLOSE,
        message.c_str());
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
}

char* moobyFileChooser(const char* message, const char* filespec, 
                        const std::string& last)
{
    char* filename = NULL;
    GtkWidget *chooser = gtk_file_chooser_dialog_new (message,
                NULL,
                GTK_FILE_CHOOSER_ACTION_OPEN,
                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                NULL);

    if (last.size()) gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(chooser),last.c_str());

    GtkFileFilter *filter = gtk_file_filter_new ();
    GtkFileFilter *filterAll = gtk_file_filter_new ();
    gtk_file_filter_add_pattern (filterAll,"*");
    gtk_file_filter_set_name (filterAll,"All Files");
    gtk_file_filter_set_name (filter,"Supported files");
    
    // Stupid fucking globbing doesn't work with brackets.
    //gtk_file_filter_add_pattern (filter,filespec);
    if (filespec)
        gtk_file_filter_add_pattern (filter,filespec);
    else
    {
        gtk_file_filter_add_pattern (filter,"*.iso");
        gtk_file_filter_add_pattern (filter,"*.bin");
        gtk_file_filter_add_pattern (filter,"*.bwi");
        gtk_file_filter_add_pattern (filter,"*.img");
        gtk_file_filter_add_pattern (filter,"*.Z");
        gtk_file_filter_add_pattern (filter,"*.Z.table");
        gtk_file_filter_add_pattern (filter,"*.bz");
        gtk_file_filter_add_pattern (filter,"*.bz.index");
    }
    gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(chooser), filterAll);
    gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(chooser), filter);
    gtk_file_chooser_set_filter( GTK_FILE_CHOOSER(chooser), filter);
    if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (chooser));
    gtk_widget_destroy (chooser);
    while (gtk_events_pending()) gtk_main_iteration();
    return filename;
}

int moobyAsk(const char* message)
{
    GtkWidget *dialog = gtk_message_dialog_new (NULL,
        GTK_DIALOG_MODAL,
        GTK_MESSAGE_QUESTION,
        GTK_BUTTONS_YES_NO,
        message);
    gint result = gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
    return result == GTK_RESPONSE_YES;
}

// given a string and a number, returns the ' ' delimited word
// at position num
std::string word(const std::string& str, const unsigned long num)
{
   if (str == std::string())
      return str;

   unsigned long i;
   std::string::size_type startpos = 0;
   std::string::size_type endpos = 0;

   for(i = 0; i < num; i++)
   {
      startpos = str.find_first_not_of(' ', endpos);
      if (startpos == std::string::npos)
         return std::string();

      endpos = str.find_first_of(' ', startpos);
      if (endpos == std::string::npos)
      {
         endpos = str.size();
      }
   }
   return str.substr(startpos, endpos - startpos);
}

// turns a string into its lowercase...
std::string tolcstr(const std::string& s)
{
   std::string::size_type i;
   std::string toReturn(s);
   for (i = 0; i < s.size(); i++)
      toReturn[i] = tolower(s[i]);
   return toReturn;
}

// tries to match the extension ext to the end of the file name
bool extensionMatches(const std::string& file, const std::string& ext)
{
   if (file.size() < ext.size()) return false;
   return (tolcstr(file.substr(file.size() - ext.size())) == ext);
}

// returns what the name of the program running this plugin is.
std::string getProgramName(void)
{
   std::string toReturn;

#ifdef _WINDOWS
   char buf[1024];
   if (GetModuleFileName(NULL, (char*)&buf, 1024) == 0)
      // i hope this never happens....  we'll just make an assumption ;)
      toReturn = "epsxe";
#else
   char* buf = getenv("_");
   if (buf == NULL)
      return "pcsx";
#endif
   else
      toReturn = buf;

	// strips all the useless crud from the returned string
   gchar* temp  = g_path_get_basename(toReturn.c_str());
   gchar* temp2 = g_ascii_strdown(temp,-1);
   toReturn = temp2;
   g_free(temp);
   g_free(temp2);
   std::string::size_type i = toReturn.find_last_of('.');
   if (i != std::string::npos)
      toReturn.erase(i);
   return tolcstr(toReturn);
}

// Crops of ALL file extensions
std::string stripFileExt(std::string str)
{
    std::string str2 = str;
    int i = str.rfind('.');
    str2 = str.substr(0,i); // Cut off file extension
    if (!str.substr(i).compare(".Z") || !str.substr(i).compare(".bz"))
    {
        // If is it a compressed file, cut off one more file extension.
        i = str.rfind('.',i-1);
        str2 = str.substr(0,i);
    }
    return str2;
}
