/************************************************************************

VirusCDR Utils.hpp

Copyright (C) 2007 Virus
Copyright (C) 2002 mooby

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

************************************************************************/

#pragma warning(disable:4786)

#ifndef UTILS_HPP
#define UTILS_HPP

#include <string.h>
#include <sstream>
#include <gtk/gtk.h>

#include <stdlib.h>
#ifdef _WINDOWS
#include <windows.h>
#endif

/* 
   This file is for any utility function or constant that has use outside the scope of any
   single file
*/

// specifies how you want the MSF time formatted
// msf/fsm -> if time is specified in minute, second, frame or backwards
// int/bcd -> integer or binary coded decimal format
enum TDTNFormat
{
   msfint,
   fsmint,
   fsmbcd,
   msfbcd
};

// a quick way to tell which API set is being used
enum EMUMode
{
   psemu,
   fpse
};

// converts uchar in c to BCD character
#define intToBCD(c) (unsigned char)((c%10) | ((c/10)<<4))

// converts BCD number in c to uchar
#define BCDToInt(c) (unsigned char)((c & 0x0F) + 10 * ((c & 0xF0) >> 4))

//static const std::string theUsualSuspects = 
//  "Common image files (*.{bz,bz.index,Z,Z.table,bin,bwi,img,iso})";

// CD time constants
static const unsigned int secondsPerMinute = 60;
static const unsigned int framesPerSecond  = 75;
static const unsigned int framesPerMinute  = framesPerSecond * secondsPerMinute;
static const unsigned int bytesPerFrame    = 2352;
static const unsigned int bytesPerSecond   = bytesPerFrame * framesPerSecond;
static const unsigned int bytesPerMinute   = bytesPerSecond * secondsPerMinute;
static const unsigned int samplesPerFrame  = bytesPerFrame/2;

void moobyMessage(const std::string& message);
char* moobyFileChooser(const char* message, const char* filespec, 
                              const std::string& last = std::string());
int moobyAsk(const char* message);
std::string word(const std::string& str, const unsigned long num);
std::string tolcstr(const std::string& s);
bool extensionMatches(const std::string& file, const std::string& ext);
std::string getProgramName(void);
std::string stripFileExt(std::string filename);

// since most binary data is stored little endian, this flips
// the data bits for big endian systems
template <class T>
inline void flipBits(T& num)
{
#if 0
   g_strreverse((char*)&num);
#endif
}


#endif
